from collections import deque


class BasicRegister:
    """The BasicRegister represents a hardware register capable of manipulating multiple bits at a time.

    :param list[bool] memory: The bits stored inside the register.
    """

    def __init__(self, memory: list[bool]):
        """Constructor method"""
        self.memory: deque[bool] = deque(memory)

    def __str__(self) -> str:
        return f"Memory: {[int(value) for value in  self.memory]}"

    def reverse(self):
        self.memory = [not value for value in self.memory]

    def left_shift(self, digit_to_fill: bool = False, steps_shifted: int = 1) -> deque[bool]:
        self.memory.extend([digit_to_fill] * steps_shifted)
        shifted_radices: deque[bool] = deque([self.memory.popleft() for _i in range(steps_shifted)])
        return shifted_radices

    def right_shift(self, digit_to_fill: bool = False, steps_shifted: int = 1) -> deque[bool]:
        self.memory.extendleft([digit_to_fill] * steps_shifted)
        shifted_radices: deque[bool] = deque([self.memory.pop() for _i in range(steps_shifted)])
        return shifted_radices


def get_memory(variable_name: str) -> list[bool]:
    """
    Reads user input to be used as a memory array.

    :param str variable_name: The name to be displayed in the input line.

    :return: A list of boolean values read from user.
    :rtype: list[bool]
    """
    while True:
        input_chars: list[str] = list(input(f"Enter {variable_name}: "))

        if all(character in ["0", "1"] for character in input_chars):
            return [True if character == "1" else False for character in input_chars]
        else:
            print(f"[ERROR] The {variable_name} may contain only 1-s and 0-s!")
